function [mu,v,nmeanv] = FindKarcherMean(q,X)

[r,N,n] = size(q);

% initialize algorithm
mu = q(:,:,1);

% plot initialization
muXinit=q_to_curve(mu);
figure(100),clf;
plot3(muXinit(1,:),muXinit(2,:),muXinit(3,:),'r','LineWidth',2)
axis equal
title('Evolution of Mean Estimate','FontSize',24)

% set parameters
lam = 0;
iter = 1;
stp = 1;
nmeanv(1) = inf;
maxiter=20;

% apply iterative algorithm
while (nmeanv(iter) > 0.01 && iter<maxiter)
    
    % compute inverse exponential map and energy gradient
    meanv = zeros(r,N);
    for i=1:n
        v(:,:,i) = FindGeodesicVector(mu,q(:,:,i),X(:,:,i),lam);
        meanv = meanv + v(:,:,i)/(n);
    end
    
    % compute magnitude of gradient
    nmeanv(iter+1) = sqrt(InnerProd_Q(meanv,meanv));
    
    % compute update using exponential map
    mu = cos(stp*nmeanv(iter+1))*mu + sin(stp*nmeanv(iter+1))*meanv/(nmeanv(iter+1));   
    muX = q_to_curve(mu);
    
    % plot update
    figure(100),clf;
    plot3(muXinit(1,:),muXinit(2,:),muXinit(3,:),'r','LineWidth',2)
    hold on
    plot3(muX(1,:),muX(2,:),muX(3,:),'LineWidth',2)
    axis equal
    title('Evolution of Mean Estimate','FontSize',24)
    
    iter = iter + 1;
end 